
-- Copyright (C) <2015>  <Josef Sedla?k>
-- 
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
-- 


function table.contains(table, element)
	for _, value in pairs(table) do
		if value == element then 
			return true;
		end
	end
	return false;
end

local L = LeriLocalization;
local function defaultFunc(L, key)
	return key;
end
setmetatable(L, {__index=defaultFunc});

LeriAddonSave = {}; -- saved variables

local LeriAddon = {};
LeriAddon.DEBUG = false
LeriAddon.INSTANCE = nil;
LeriAddon.CACHE_TIMEOUT = 86400 * 7; -- cache expiration - 7 days

LeriAddon.log = function(...)
	if LeriAddon.DEBUG then
		local text = "";
		for i = 1, select("#", ...) do
			if strlen(text) > 0 then
				text = text .. " ,  ";
			end
			local v = select(i, ...);
			text = text .. tostring(v);
		end
		print(text);
	end
end

LeriAddon.clearOldCache = function()
	if LeriAddonSave then
		local curTime = time();
		for role, players in pairs(LeriAddonSave) do
			for key, player in pairs(players) do
				if player.time + LeriAddon.CACHE_TIMEOUT < curTime then
					table.remove(players, key);
				end
			end
		end
	end  
end

local Leri = {};
Leri.__index = Leri;
LeriAddon.Leri = Leri;

-- create instance of Leri addon
function Leri:Create()
	local self = {};
	setmetatable(self, Leri);

	self._openWindow = false;
	self._queue = {}; -- player queue for update 
	self._player = nil; -- current player info
	self._member = nil; -- current scanned player's index
	self._unitId = nil; -- UnitId
	self._slot = nil; -- slot with leg. ring for current scanned player
	self._isScanning = false; -- is scanning run?
	self._isGroup = false; -- is player in party/raid?
	self._eventFrame = CreateFrame("Frame");   
	self._eventFrame:SetScript("OnEvent", function(frame, event) self:eventHandler(frame, event); end);

	self._players = {}; -- players in raid/party... with legendary ring

	self:_initRoles();
	self:_initRings();

	LeriAddon.INSTANCE = self;
end

function Leri:setOpen(open)
	self._openWindow = open;
end

function Leri:_getUnitName(unitId)
	return GetUnitName(unitId, true);
end

function Leri:_getCachedPlayer(role, playerName)
	local result = nil;
	if LeriAddonSave then
		if LeriAddonSave[role] then 
			for _, player in pairs(LeriAddonSave[role]) do
				if player then
					if player.name == playerName then
						result = player;
						break; 
					end
				end
			end
		end
	end  

	return result;
end

function Leri:eventHandler(frame, event, other)   
	frame:UnregisterEvent(event);
	LeriAddon.log("eventHandler", event);
	if event == "INSPECT_READY" then
		self:_inspectReadyHandler();
	elseif event == "GET_ITEM_INFO_RECEIVED" then
		self:_itemInfoHandler();
	elseif event == "ADDON_LOADED" then
		LeriAddon.clearOldCache();
	elseif event == "UNIT_INVENTORY_CHANGED" then
		self:_updatePlayer(other); 
	elseif event == "RAID_ROSTER_UPDATE" or event == "PARTY_MEMBERS_CHANGED" then
		self:scan();
	end
end

function Leri:_removePlayer(name)
	if collection then
		for role, players in pairs(self._players[role]) do
			print(role, players);
			for key, player in pairs(players) do
				print(key, player);
				if player.name == name then
					table.remove(collection[role], key);
					break;
				end
			end
		end
	end
end

function Leri:_updatePlayer(other)
	if LeriAddonSave then
		local name = self:_getUnitName(other);
		self:_remove(LeriAddonSave, name);
	end
	self:scan();
end

function Leri:_initRoles()
	self.ROLE_TANK = "TANK";
	self.ROLE_HEAL = "HEALER";
	self.ROLE_DMG = "DAMAGER";   
end

function Leri:_initRings()
	self._rings = {};   
	self._rings[self.ROLE_TANK] = {124637};
	self._rings[self.ROLE_HEAL] = {124638};
	self._rings[self.ROLE_DMG] = {124636, 124635, 124634};   
end

-- start scanning players
function Leri:scan()
	if self._isScanning then
		return;
	end   

	self._isScanning = true;
	self:_resetPlayers();

	self._isGroup = self:_getNumMembers() > 1;
	self._member = 0;
	self:_scanNext();
end

-- reset current players with leg. ring
function Leri:_resetPlayers()
	self._players[self.ROLE_TANK] = {};
	self._players[self.ROLE_HEAL] = {};
	self._players[self.ROLE_DMG] = {};
end

-- get party/raid num members
function Leri:_getNumMembers()
	local numMembers = 1;
	if UnitInRaid("player") or UnitInParty("player") then
		numMembers = GetNumGroupMembers();
	end

	return numMembers;
end

function Leri:_getUnitId(index) 
	local unitId = IsInRaid() and "raid"..index or "party"..index;   
	local name = self:_getUnitName(unitId);
	if name == nil then
		unitId = "player";
	end

	return unitId;
end

function Leri:_inspectReadyHandler()
	LeriAddon.log("_inspectReadyHandler");
	local name = self:_getUnitName(self._unitId);

	if UnitExists(self._unitId) then
		local role;
		if self._isGroup then
			role = UnitGroupRolesAssigned(name);
		else
			local specId = GetSpecialization();
			LeriAddon.log("specId: ", specId);
			role = specId and select(6, GetSpecializationInfo(specId)) or nil;
		end

		local ringIds = self._rings[role];

		self._slot = INVSLOT_FINGER1;
		local ring = GetInventoryItemID(self._unitId, self._slot);
		hasLegendRing = table.contains(ringIds, ring);
		if not hasLegendRing then   
			self._slot = INVSLOT_FINGER2;
			ring = GetInventoryItemID(self._unitId, self._slot);   
			hasLegendRing = table.contains(ringIds, ring);
		end

		local ringIds = self._rings[role];

		self._player = {};
		self._player.name = name;
		self._player.role = role;

		if hasLegendRing then
			local link = GetInventoryItemLink(self._unitId, self._slot);
			if link == nil then
				LeriAddon.log("no link");               
				self._player.ring = 0;   
				self:_savePlayer(false);
				self._slot = nil;
				ClearInspectPlayer(self._unitId);   
				self:_scanNext();
			else
				local _, itemLink, _, iLvl = GetItemInfo(link);
				if itemLink then
					self._player.ring = iLvl;
					LeriAddon.log(self._player.name .. " has legendary ring(" .. self._player.ring .. ")");            
					self:_savePlayer(true);
					self._slot = nil;
					ClearInspectPlayer(self._unitId);   
					self:_scanNext();
				end         
			end
		else
			self._slot = nil;
			ClearInspectPlayer(self._unitId);   
			self:_scanNext();
		end
	else
		self._slot = nil;
		ClearInspectPlayer(self._unitId);   
		self:_scanNext();
	end
end

function Leri:_showPlayers(role, tabPage)
	local playerList = self._players[role];

	local info = "";
		for _, player in pairs(playerList) do
			info = info .. "  " .. player.name;
			if player.ring and player.ring > 0 then
				info = info .. " - " .. player.ring .. "\n";
			else
				info = info .. "\n";
			end         
		end

	tabPage:SetText(info);
end

function Leri:_showResult() 
	leriStatus:SetText(L["STATUS_DONE"]);
	self:_showPlayers(self.ROLE_DMG, leriDpsFrame);
	self:_showPlayers(self.ROLE_HEAL, leriHealFrame);
	self:_showPlayers(self.ROLE_TANK, leriTankFrame);
end

-- scan next player
function Leri:_scanNext()
	LeriAddon.log("_scanNext");
	if self._member == nil then 
		return;
	end

	local fromUpdateQueue = false;
	local partyMembers = self:_getNumMembers();
	self._member = self._member + 1;
	if self._member > partyMembers then
		if table.getn(self._queue) > 0 then
			fromUpdateQueue = true;
			self._unitId = self._queue[0];
			table.remove(self._queue, 0);
		else    
			self._member = nil;
			self._isScanning = false;
			if self._openWindow then
				self:_showResult();
			end
			return;
		end
	end

	if not fromUpdateQueue then
		self._unitId = self:_getUnitId(self._member);
	end

	LeriAddon.log(self._unitId);
	local name = self:_getUnitName(self._unitId);
	if isGroup then
		role = UnitGroupRolesAssigned(name);
	else
		local specId = GetSpecialization();
		role = specId and select(6, GetSpecializationInfo(specId)) or nil;
	end
	LeriAddon.log("name", name);

	leriStatus:SetText(L["STATUS_SCANNING"] .. self._member .. " / " .. partyMembers .. " (" .. name .. ")");

	self._player = self:_getCachedPlayer(role, name);
	if self._player then
		LeriAddon.log("player from cache");
		self:_savePlayer(false);
		self:_scanNext();
	else
		LeriAddon.log("player not in cache");
		local hasLegendRing = false;
		if UnitExists(self._unitId) then
			if CheckInteractDistance(self._unitId, 1) and CanInspect(self._unitId) then
				self._eventFrame:RegisterEvent("INSPECT_READY");
				NotifyInspect(self._unitId);
			else
				self:_scanNext();
			end         
		else
			self:_scanNext();
		end   
	end
end

function Leri:_savePlayer(toCache)
	LeriAddon.log("savePlayer", self._player.name);
	local role = self._player.role;
	self:_removePlayer(self._player.name);
	table.insert(self._players[role], self._player);
	if toCache then
		self._player.time = time();
		if not LeriAddonSave then
			LeriAddonSave = {};
		end
		if not LeriAddonSave[role] then
			LeriAddonSave[role] = {};
		end 
		table.insert(LeriAddonSave[role], self._player);
	end 
end

function Leri:_itemInfoHandler()
	LeriAddon.log("_getCachedPlayer");
	local link = GetInventoryItemLink(self._unitId, self._slot);

	if link == nil then
		self._player.ring = 0;            
	else
		local _, itemLink, _, iLvl = GetItemInfo(link);
		if itemLink == nil then
			leriAddon.log(name .. " has legendary ring(" .. iLvl .. ")");
			self._player.ring = iLvl;
		end
	end

	self:_savePlayer(true);

	self._slot = nil;
	ClearInspectPlayer(self._unitId);   
	self:_scanNext();
end

function Leri:printToChat()
	local chatType;
	if UnitInRaid("player") then
		chatType = "RAID";
	elseif  UnitInParty("player") then
		chatType = "PARTY";
	else
		chatType = "WHISPER";
	end

	if chatType == "WHISPER" then
		self:_send2ChatFrame()
	else
		self:_send2Raid(chatType)
	end
end   

function Leri:_send2Raid(chatType) 
	local msg = "Legendary ring";
	SendChatMessage(msg, chatType, nil, "Risaya");
	for role, players in pairs(self._players) do
		if table.getn(players) > 0 then
			SendChatMessage(role, chatType, nil, "Risaya");
			for _, player in pairs(players) do
				local msg = player.name;
				if player.ring > 0 then
					msg = msg .. " - " .. player.ring;
				end
				SendChatMessage(msg, chatType, nil, "Risaya");
			end
		end      
	end
end

function Leri:_send2ChatFrame()
	local msg = "Legendary ring";
	for role, players in pairs(self._players) do
		if table.getn(players) > 0 then
			msg = msg .. "\n  " .. role .. ":\n";
			for _, player in pairs(players) do                                       
				msg = msg .. "  " .. player.name;
				if player.ring > 0 then
					msg = msg .. " - " .. player.ring;
				end
				msg = msg .. "\n";
			end

			ChatFrame1:AddMessage(msg);
		end      
	end   
end


-- contains addon class instance
local function open(command)
	LeriAddon.log("**********************************");
	if _G["LeriAddon"] == nil then
		_G["LeriAddon"] = LeriAddon;
		_G["LeriAddon"].Leri.Create(_G);

		leriTitle:SetText(L["FRAME_TITLE"]);
		leriSendBtn:SetText(L["FRAME_TAB_SEND_BTN"]);
		leriFrameTab1:SetText(L["FRAME_TAB_DPS"]);
		leriFrameTab2:SetText(L["FRAME_TAB_HEAL"]);
		leriFrameTab3:SetText(L["FRAME_TAB_TANK"]);
		leriSendBtn:SetScript("OnClick", function() _G["LeriAddon"].INSTANCE:printToChat(); end);
	end


	if command == "clean" then
		LeriAddonSave = {};
		LeriAddon.clearOldCache(); 
		print(L["CACHE_CLEANED"]);
	end

	if command == "open" then
		leriFrame:Show();
		_G["LeriAddon"].INSTANCE:setOpen(true);
		_G["LeriAddon"].INSTANCE:scan();
	end

	if command == "" then
		print(L["HELP_LERI"]);
		print(L["HELP_LERI_OPEN"])
		print(L["HELP_LERI_CLEAN"])
	end   
end

SLASH_LERI1 = '/leri';
SlashCmdList["LERI"] = open;